<?php
	/**
	 * This is a quick-and-dirty draft QPanel object to do Create, Edit, and Delete functionality
	 * of the ProductImage class.  It uses the code-generated
	 * ProductImageMetaControl class, which has meta-methods to help with
	 * easily creating/defining controls to modify the fields of a ProductImage columns.
	 *
	 * Any display customizations and presentation-tier logic can be implemented
	 * here by overriding existing or implementing new methods, properties and variables.
	 * 
	 * NOTE: This file is overwritten on any code regenerations.  If you want to make
	 * permanent changes, it is STRONGLY RECOMMENDED to move both product_image_edit.php AND
	 * product_image_edit.tpl.php out of this Form Drafts directory.
	 *
	 * @package Quasi
	 * @subpackage Drafts
	 */
	class ProductImageEditPanel extends QPanel {
		// Local instance of the ProductImageMetaControl
		protected $mctProductImage;

		// Controls for ProductImage's Data Fields
		public $lblId;
		public $lstProduct;
		public $txtTitle;
		public $txtAltTag;
		public $txtDescription;
		public $txtUri;
		public $txtXSize;
		public $txtYSize;
		public $lstSizeTypeObject;

		// Other ListBoxes (if applicable) via Unique ReverseReferences and ManyToMany References

		// Other Controls
		public $btnSave;
		public $btnDelete;
		public $btnCancel;

		// Callback
		protected $strClosePanelMethod;

		public function __construct($objParentObject, $strClosePanelMethod, $intId = null, $strControlId = null) {
			// Call the Parent
			try {
				parent::__construct($objParentObject, $strControlId);
			} catch (QCallerException $objExc) {
				$objExc->IncrementOffset();
				throw $objExc;
			}

			// Setup Callback and Template
			$this->strTemplate = 'ProductImageEditPanel.tpl.php';
			$this->strClosePanelMethod = $strClosePanelMethod;

			// Construct the ProductImageMetaControl
			// MAKE SURE we specify "$this" as the MetaControl's (and thus all subsequent controls') parent
			$this->mctProductImage = ProductImageMetaControl::Create($this, $intId);

			// Call MetaControl's methods to create qcontrols based on ProductImage's data fields
			$this->lblId = $this->mctProductImage->lblId_Create();
			$this->lstProduct = $this->mctProductImage->lstProduct_Create();
			$this->txtTitle = $this->mctProductImage->txtTitle_Create();
			$this->txtAltTag = $this->mctProductImage->txtAltTag_Create();
			$this->txtDescription = $this->mctProductImage->txtDescription_Create();
			$this->txtUri = $this->mctProductImage->txtUri_Create();
			$this->txtXSize = $this->mctProductImage->txtXSize_Create();
			$this->txtYSize = $this->mctProductImage->txtYSize_Create();
			$this->lstSizeTypeObject = $this->mctProductImage->lstSizeTypeObject_Create();

			// Create Buttons and Actions on this Form
			$this->btnSave = new QButton($this);
			$this->btnSave->Text = QApplication::Translate('Save');
			$this->btnSave->AddAction(new QClickEvent(), new QAjaxControlAction($this, 'btnSave_Click'));
			$this->btnSave->CausesValidation = $this;

			$this->btnCancel = new QButton($this);
			$this->btnCancel->Text = QApplication::Translate('Cancel');
			$this->btnCancel->AddAction(new QClickEvent(), new QAjaxControlAction($this, 'btnCancel_Click'));

			$this->btnDelete = new QButton($this);
			$this->btnDelete->Text = QApplication::Translate('Delete');
			$this->btnDelete->AddAction(new QClickEvent(), new QConfirmAction(QApplication::Translate('Are you SURE you want to DELETE this') . ' ' . QApplication::Translate('ProductImage') . '?'));
			$this->btnDelete->AddAction(new QClickEvent(), new QAjaxControlAction($this, 'btnDelete_Click'));
			$this->btnDelete->Visible = $this->mctProductImage->EditMode;
		}

		// Control AjaxAction Event Handlers
		public function btnSave_Click($strFormId, $strControlId, $strParameter) {
			// Delegate "Save" processing to the ProductImageMetaControl
			$this->mctProductImage->SaveProductImage();
			$this->CloseSelf(true);
		}

		public function btnDelete_Click($strFormId, $strControlId, $strParameter) {
			// Delegate "Delete" processing to the ProductImageMetaControl
			$this->mctProductImage->DeleteProductImage();
			$this->CloseSelf(true);
		}

		public function btnCancel_Click($strFormId, $strControlId, $strParameter) {
			$this->CloseSelf(false);
		}

		// Close Myself and Call ClosePanelMethod Callback
		protected function CloseSelf($blnChangesMade) {
			$strMethod = $this->strClosePanelMethod;
			$this->objForm->$strMethod($blnChangesMade);
		}
	}
?>